package com.lumberyard.graph.neo4j.plugins.communityclustering;

import static org.junit.Assert.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

public class DeltaModularityMatrixTest {
    @Test
    public void testAddEdgeForExistingNode_edgesAddedCorrectly() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();

        assertEquals(3, q.getAllRows().size());
        assertEquals(2, q.getRow(SimpleTestGraph.node1Id).size());
        assertTrue(q.getRow(SimpleTestGraph.node1Id).containsKey(SimpleTestGraph.node2Id));
        assertTrue(q.getRow(SimpleTestGraph.node1Id).containsKey(SimpleTestGraph.node3Id));
    }

    @Test
    public void testMaxDeltaModularityForRow_oneEdge_sameEdgeReturned() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();

        assertEquals(SimpleTestGraph.node2Id, q.getMaxDeltaModularityForRow(SimpleTestGraph.node1Id).getCommunityId());
        assertEquals(0.125, q.getMaxDeltaModularityForRow(SimpleTestGraph.node1Id).getDeltaModularity(), 0.000001);
    }

    @Test
    public void testMaxDeltaModularityForRow_twoEdges_communityWithHighestDegreeReturned() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();

        assertEquals(SimpleTestGraph.node2Id, q.getMaxDeltaModularityForRow(SimpleTestGraph.node1Id).getCommunityId());
    }

    @Test
    public void testInitFractionOfEdgeEndsAttachedToCommunity_correctFractionCalculatedForEachNode() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();

        assertEquals(0.5, q.getFractionOfEdgeEndsAttachedToCommunity().get(SimpleTestGraph.node1Id), 0.000001);
        assertEquals(0.25, q.getFractionOfEdgeEndsAttachedToCommunity().get(SimpleTestGraph.node2Id), 0.000001);
        assertEquals(0.25, q.getFractionOfEdgeEndsAttachedToCommunity().get(SimpleTestGraph.node3Id), 0.000001);
    }

    @Test
    public void testJoinCommunities_trivialCase_communitiesJoinedAndOneOfJoinedCommunitiesRemoved() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();
        q.joinCommunities(SimpleTestGraph.node1Id, SimpleTestGraph.node2Id);

        assertFalse(q.getAllRows().contains(SimpleTestGraph.node1Id));
        assertFalse(q.getRow(SimpleTestGraph.node3Id).containsKey(SimpleTestGraph.node1Id));
    }

    @Test
    public void testJoinCommunities_trivialCase_communitiesJoinedAndMerged() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();
        q.joinCommunities(SimpleTestGraph.node1Id, SimpleTestGraph.node2Id);

        assertEquals(2, q.getAllRows().size());
        assertTrue(q.getAllRows().contains(SimpleTestGraph.node2Id));
        assertTrue(q.getAllRows().contains(SimpleTestGraph.node3Id));
        assertEquals(1, q.getRow(SimpleTestGraph.node3Id).size());
        assertTrue(q.getRow(SimpleTestGraph.node3Id).containsKey(SimpleTestGraph.node2Id));
        assertEquals(1, q.getRow(SimpleTestGraph.node2Id).size());
        assertTrue(q.getRow(SimpleTestGraph.node2Id).containsKey(SimpleTestGraph.node3Id));
    }

    @Test
    public void testJoinCommunities_trivialCase_fractionsOfEdgeEndsUpdatedCorrectly() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();
        q.joinCommunities(SimpleTestGraph.node1Id, SimpleTestGraph.node2Id);

        assertEquals(0.0, q.getFractionOfEdgeEndsAttachedToCommunity().get(SimpleTestGraph.node1Id), 0.000001);
        assertEquals(0.75, q.getFractionOfEdgeEndsAttachedToCommunity().get(SimpleTestGraph.node2Id), 0.000001);
        assertEquals(0.25, q.getFractionOfEdgeEndsAttachedToCommunity().get(SimpleTestGraph.node3Id), 0.000001);
    }

    @Test
    public void testJoinCommunities_trivialCase_deltaModularitiesForFirstCommunityRemoved() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();
        q.joinCommunities(SimpleTestGraph.node1Id, SimpleTestGraph.node2Id);

        assertNull(q.getMaxDeltaModularityForRow(SimpleTestGraph.node1Id));
        // assertEquals(-0.25,
        // q.getRow(SimpleTestGraph.node2Id).get(SimpleTestGraph.node3Id).getDeltaModularity(),
        // 0.000001);
        // assertEquals(-0.25,
        // q.getRow(SimpleTestGraph.node3Id).get(SimpleTestGraph.node2Id).getDeltaModularity(),
        // 0.000001);
    }

    @Test
    public void testJoinCommunities_trivialCase_deltaModularityMaxHeapsUpdatedCorrectly() {
        DeltaModularityMatrix q = initDeltaModularityMatrixFromSimpleTestData();
        q.joinCommunities(SimpleTestGraph.node1Id, SimpleTestGraph.node2Id);

//        assertEquals(SimpleTestGraph.node3Id, q.getMaxDeltaModularityForRow(SimpleTestGraph.node2Id).getCommunityId());
        assertEquals(SimpleTestGraph.node2Id, q.getMaxDeltaModularityForRow(SimpleTestGraph.node3Id).getCommunityId());
//        assertEquals(-0.0625, q.getMaxDeltaModularityForRow(SimpleTestGraph.node2Id).getDeltaModularity(), 0.000001);
        assertEquals(-0.0625, q.getMaxDeltaModularityForRow(SimpleTestGraph.node3Id).getDeltaModularity(), 0.000001);
    }

    private DeltaModularityMatrix initDeltaModularityMatrixFromSimpleTestData() {
        DeltaModularityMatrix q = new DeltaModularityMatrix();

        q.addEdge(SimpleTestGraph.node1Id, SimpleTestGraph.node1Degree, SimpleTestGraph.node2Id, SimpleTestGraph.node2Degree,
                SimpleTestGraph.totalEdges);
        q.addEdge(SimpleTestGraph.node1Id, SimpleTestGraph.node1Degree, SimpleTestGraph.node3Id, SimpleTestGraph.node3Degree,
                SimpleTestGraph.totalEdges);
        q.initFractionOfEdgeEndsAttachedToCommunity(SimpleTestGraph.totalEdges);
        return q;
    }
}
